@@ -99,6 +99,11 @@ main(int argc, char ** argv)
 # include <kvm.h>
 #endif
 
+#ifdef SYS_VXWORKS
+/* vxWorks needs mode flag -casey*/
+#define open(name, flags)   open(name, flags, 0777)
+#endif
+
 #ifndef L_SET	/* Was: defined(SYS_PTX) || defined(SYS_IX86OSF1) */
 # define L_SET SEEK_SET
 #endif
@@ -121,8 +126,8 @@ int writetick = 0;
 int quiet = 0;
 int setnoprintf = 0;
 
-char *kmem = KMEM;
-char *file = NULL;
+const char *kmem = KMEM;
+const char *file = NULL;
 int   fd  = -1;
 
 static	void	getoffsets	P((unsigned long *, unsigned long *, unsigned long *, unsigned long *));
@@ -585,7 +590,7 @@ getoffsets(tick_off, tickadj_off, dosync_off, noprintf_off)
     NL_B "" NL_E,
   };
 
-  static char *kernels[] =
+  static const char *kernels[] =
   {
 #ifdef HAVE_GETBOOTFILE
     NULL,			/* *** SEE BELOW! *** */
@@ -595,7 +600,7 @@ getoffsets(tick_off, tickadj_off, dosync_off, noprintf_off)
     "/vmunix",
     "/unix",
     "/mach",
-    "hp-ux",
+    "/hp-ux",
     "/386bsd",
     "/netbsd",
     "/stand/vmunix",
