@@ -3,6 +3,7 @@
  */
 #include <stdio.h>
 #include <ctype.h>
+#include <sys/param.h>
 #include <sys/types.h>
 #include <signal.h>
 #if !defined(VMS)
@@ -156,7 +157,7 @@
  * Translation table - keywords to function index
  */
 struct keyword {
-	char *text;
+	const char *text;
 	int keytype;
 };
 
@@ -298,7 +299,7 @@ static struct keyword flags_keywords[] = {
  * "logconfig" building blocks
  */
 struct masks {
-  char         *name;
+  const char   *name;
   unsigned long mask;
 };
 
@@ -364,7 +365,7 @@ extern int debug;
 #endif
 	char	*progname;
 	char	sys_phone[MAXPHONE][MAXDIAL]; /* ACTS phone numbers */
-static	char	*xntp_options = "aAbc:dD:f:gk:l:mp:r:s:t:v:V:x";
+static	const char	*xntp_options = "aAbc:dD:f:gk:l:mp:r:s:t:v:V:x";
 
 #ifdef SYS_WINNT
 extern HANDLE ResolverThreadHandle;
@@ -379,7 +380,7 @@ static  unsigned long get_logmask P((char *));
 static	int	gettokens	P((FILE *, char *, char **, int *));
 static	int	matchkey	P((char *, struct keyword *));
 static	int	getnetnum	P((char *, struct sockaddr_in *, int));
-static	void	save_resolve	P((char *, int, int, int, int, int, int, u_long));
+static	void	save_resolve	P((char *, int, int, int, int, int, int, u_int32));
 static	void	do_resolve_internal P((void));
 static	void	abort_resolve	P((void));
 #if !defined(VMS)
@@ -569,7 +570,8 @@ getconfig(argc, argv)
 	int minpoll;
 	int maxpoll;
 	int ttl;
-	u_long peerkey;
+	u_int32 peerkey;
+	u_long lpeerkey;
 	int peerflags;
 	int hmode;
 	struct sockaddr_in peeraddr;
@@ -592,7 +594,7 @@ getconfig(argc, argv)
 	extern char *ntp_optarg;
 	extern char *Version;
 	extern char *set_tod_using;
-	extern u_long info_auth_keyid;
+	extern u_int32 info_auth_keyid;
 	extern int allow_set_backward;
 	extern int correct_any;
 	FILEGEN *filegen;
@@ -725,7 +727,7 @@ getconfig(argc, argv)
 			
 		case 't':
 			do {
-				u_long tkey;
+				u_int32 tkey;
 				
 				tkey = atol(ntp_optarg);
 				if (tkey <= 0 || tkey > NTP_MAXKEY) {
@@ -762,26 +764,28 @@ getconfig(argc, argv)
 		exit(2);
 	}
 
-	if ((fp = fopen(FindConfig(config_file), "r")) == NULL) {
-		
+	if ((fp = fopen(FindConfig(config_file), "r")) == NULL)
+	  {	
+	    fprintf(stderr, "getconfig: Couldn't open <%s>\n", FindConfig(config_file));
+	    msyslog(LOG_INFO, "getconfig: Couldn't open <%s>", FindConfig(config_file));
 #ifdef SYS_WINNT
-		/* Under WinNT try alternate_config_file name, first NTP.CONF, then NTP.INI */
+	    /* Under WinNT try alternate_config_file name, first NTP.CONF, then NTP.INI */
 
-		fclose(fp);
-		if ((fp = fopen(FindConfig(alt_config_file), "r")) == NULL) {
+	    if ((fp = fopen(FindConfig(alt_config_file), "r")) == NULL) {
         
-			/*
-			 * Broadcast clients can sometimes run without
-			 * a configuration file.
-			 */
-
-			return;
-		}
-#else
-		return;
-#endif /* SYS_WINNT */
-	}
+	      /*
+	       * Broadcast clients can sometimes run without
+	       * a configuration file.
+	       */
 
+	      fprintf(stderr, "getconfig: Couldn't open <%s>\n", FindConfig(alt_config_file));
+	      msyslog(LOG_INFO, "getconfig: Couldn't open <%s>", FindConfig(alt_config_file));
+	      return;
+	    }
+#else  /* not SYS_WINNT */
+	    return;
+#endif /* not SYS_WINNT */
+	  }
 	     
 	while ((tok = gettokens(fp, line, tokens, &ntokens))
 	       != CONFIG_UNKNOWN) {
@@ -1063,7 +1067,7 @@ getconfig(argc, argv)
 			peerversion = 0;
 			peerkey = 0;
 			errflg = 0;
-			maskaddr.sin_addr.s_addr = ~ (u_long) 0;
+			maskaddr.sin_addr.s_addr = ~ (u_int32) 0;
 			for (i = 2; i < ntokens; i++) {
 				switch (matchkey(tokens[i], res_keywords)) {
 				case CONF_RES_MASK:
@@ -1150,7 +1154,7 @@ getconfig(argc, argv)
 			
 		case CONFIG_TRUSTEDKEY:
 			for (i = 1; i < ntokens; i++) {
-				u_long tkey;
+				u_int32 tkey;
 				
 				tkey = atol(tokens[i]);
 				if (tkey == 0) {
@@ -1188,8 +1192,8 @@ getconfig(argc, argv)
 			
 		case CONFIG_CONTROLKEY:
 			if (ntokens >= 2) {
-				u_long ckey;
-				extern u_long ctl_auth_keyid;
+				u_int32 ckey;
+				extern u_int32 ctl_auth_keyid;
 				
 				ckey = atol(tokens[1]);
 				if (ckey == 0) {
@@ -1348,14 +1352,16 @@ getconfig(argc, argv)
 				case CONF_FDG_FLAG2:
 				case CONF_FDG_FLAG3:
 				case CONF_FDG_FLAG4:
-					if (!atouint(tokens[++i], &peerkey)
-					    || peerkey > 1) {
+					if (!atouint(tokens[++i], &lpeerkey)
+					    || lpeerkey > 1) {
 						msyslog(LOG_ERR,
 						    "fudge %s flag value in error",
 						    ntoa(&peeraddr));
+						peerkey = lpeerkey;
 						errflg = i;
 						break;
 					}
+					peerkey = lpeerkey;
 					switch(c) {
 					case CONF_FDG_FLAG1:
 						c = CLK_FLAG1;
@@ -1518,6 +1524,7 @@ getconfig(argc, argv)
 				    "no value for clientlimit command - line ignored");
 			} else {
 				u_long i;
+
 				if (!atouint(tokens[1], &i) || !i) {
 					msyslog(LOG_ERR,
 				           "illegal value for clientlimit command - line ignored");
@@ -1799,12 +1806,12 @@ getnetnum(num, addr, complain)
 	if (i < 4) {
 		if (complain)
 			msyslog(LOG_ERR,
-		    "configure: \"%s\" not valid host number, line ignored",
+		    "getnetnum: \"%s\" invalid host number, line ignored",
 			    num);
 #ifdef DEBUG
 		if (debug > 3)
 		        printf(
-		    "configure: \"%s\" not valid host number, line ignored\n",
+		    "getnetnum: \"%s\" invalid host number, line ignored\n",
 		            num);
 #endif
 		return 0;
@@ -1839,9 +1846,9 @@ int sig;
 	 * it should have already exited.  Hence the following
 	 * shouldn't hang.  If it does, please tell me.
 	 */
-#ifndef SYS_WINNT
+#if !defined (SYS_WINNT) && !defined(SYS_VXWORKS)
 	(void) wait(0);
-#endif /* SYS_WINNT */
+#endif /* SYS_WINNT  && VXWORKS*/
 }
 #endif /* VMS */
 
@@ -1858,8 +1865,9 @@ save_resolve(name, mode, version, minpoll, maxpoll, flags, ttl, keyid)
 	int maxpoll;
 	int flags;
 	int ttl;
-	u_long keyid;
+	u_int32 keyid;
 {
+#ifndef SYS_VXWORKS
 	if (res_fp == NULL) {
 #ifndef SYS_WINNT
 		(void) strcpy(res_file, RES_TEMPFILE);
@@ -1874,14 +1882,23 @@ save_resolve(name, mode, version, minpoll, maxpoll, flags, ttl, keyid)
 			(void) strcat(res_file, "xntpdXXXXXX");
 		}
 #endif /* SYS_WINNT */
+#ifdef HAVE_MKSTEMP
+		{
+			int fd;
+
+			res_fp = NULL;
+			if ((fd = mkstemp(res_file)) != -1)
+				res_fp = fdopen(fd, "w");
+		}
+#else
 		(void) mktemp(res_file);
 		res_fp = fopen(res_file, "w");
+#endif
 		if (res_fp == NULL) {
 			msyslog(LOG_ERR, "open failed for %s: %m", res_file);
 			return;
 		}
 	}
-
 #ifdef DEBUG
 	if (debug) {
 	       printf("resolving %s\n", name);
@@ -1889,7 +1906,11 @@ save_resolve(name, mode, version, minpoll, maxpoll, flags, ttl, keyid)
 #endif
 
 	(void) fprintf(res_fp, "%s %d %d %d %d %d %d %lu\n", name, mode,
-	    version, minpoll, maxpoll, flags, ttl, keyid);
+	    version, minpoll, maxpoll, flags, ttl, (u_long)keyid);
+	/* we don't open resolve file Casey */
+#else  /* SYS_VXWORKS */
+	/* save resolve info to a struct */
+#endif /* SYS_VXWORKS */
 }
 
 
@@ -1909,11 +1930,13 @@ abort_resolve()
 		(void) fclose(res_fp);
 	res_fp = NULL;
 
+#ifndef SYS_VXWORKS         /* we don't open the file to begin with */
 #if !defined(VMS)
 	(void) unlink(res_file);
 #else
 	(void) delete(res_file);
 #endif /* VMS */
+#endif /* SYS_VXWORKS */
 }
 
 
@@ -1935,10 +1958,10 @@ do_resolve_internal()
 	int i;
 
 #if !defined(VMS)
-	extern u_long req_keyid;	/* request keyid */
+	extern u_int32 req_keyid;	/* request keyid */
 	extern char *req_file;		/* name of the file with config info */
 #endif
-	extern u_long info_auth_keyid;
+	extern u_int32 info_auth_keyid;
 
 	if (res_fp == NULL) {
 		/* belch */
@@ -1951,7 +1974,7 @@ do_resolve_internal()
 	(void) fclose(res_fp);
 	res_fp = NULL;
 
-#if !defined(VMS)
+#if !defined(VMS) && !defined (SYS_VXWORKS)
 	/* find a keyid */
 	if (info_auth_keyid == 0)
 		req_keyid = 65535;
@@ -1987,6 +2010,7 @@ do_resolve_internal()
 #ifndef SYS_WINNT
 	(void) signal_no_reset(SIGCHLD, catchchild);
 
+#ifndef SYS_VXWORKS
 	i = fork();
 	if (i == 0) {
 		/*
@@ -2036,6 +2060,11 @@ do_resolve_internal()
 		abort_resolve();
 		exit(1);
 	}
+#else
+     /* vxWorks spawns a thread... -casey */
+     i = sp (ntp_intres);
+     /*i = taskSpawn("ntp_intres",100,VX_FP_TASK,20000,ntp_intres);*/
+#endif
 	if (i == -1) {
 		msyslog(LOG_ERR, "fork() failed, can't start ntp_intres");
 		(void) signal_no_reset(SIGCHLD, SIG_DFL);
@@ -2061,9 +2090,9 @@ do_resolve_internal()
 		}
 	}
 #endif /* SYS_WINNT */
-#else /* VMS */
+#else /* VMS  VXWORKS*/
 	msyslog(LOG_ERR, 
 	    "Name resolution not implemented for VMS - use numeric addresses");
 	abort_resolve();
-#endif /* VMS */
+#endif /* VMS  VXWORKS*/
 }
