@@ -78,8 +78,8 @@ struct conf_entry *confentries = NULL;
 
 #define	SLEEPTIME	2
 
-static	VOLATILE int config_timer = 0;
-static	VOLATILE int resolve_timer = 0;
+static	volatile int config_timer = 0;
+static	volatile int resolve_timer = 0;
 
 static	int resolve_value;	/* next value of resolve timer */
 
@@ -122,7 +122,7 @@ static	int sockfd = -1;
 
 /* stuff to be filled in by caller */
 
-u_long req_keyid;	/* request keyid */
+u_int32 req_keyid;	/* request keyid */
 char *req_file;		/* name of the file with configuration info */
 
 /* end stuff to be filled in */
@@ -136,10 +136,10 @@ extern int errno;
 static	RETSIGTYPE bong		P((int));
 static	void	checkparent	P((void));
 static	void	removeentry	P((struct conf_entry *));
-static	void	addentry	P((char *, int, int, int, int, int, int, u_long));
-static	int		findhostaddr	P((struct conf_entry *));
+static	void	addentry	P((char *, int, int, int, int, int, int, u_int32));
+static	int	findhostaddr	P((struct conf_entry *));
 static	void	openntp		P((void));
-static	int		request		P((struct conf_peer *));
+static	int	request		P((struct conf_peer *));
 static	char *	nexttoken	P((char **));
 static	void	readconf	P((FILE *, char *));
 static	void	doconfigure	P((int));
@@ -278,7 +278,7 @@ int sig;
 static void
 checkparent()
 {
-#ifndef SYS_WINNT
+#if !defined (SYS_WINNT) && !defined (SYS_VXWORKS)
 
 	/*
 	 * If our parent (the server) has died we will have been
@@ -288,7 +288,7 @@ checkparent()
 		msyslog(LOG_INFO, "parent died before we finished, exiting");
 		exit(0);
 	}
-#endif /* SYS_WINNT */
+#endif /* SYS_WINNT && SYS_VXWORKS*/
 }
 
 
@@ -330,7 +330,7 @@ addentry(name, mode, version, minpoll, maxpoll, flags, ttl, keyid)
 	int maxpoll;
 	int flags;
 	int ttl;
-	u_long keyid;
+	u_int32 keyid;
 {
 	register char *cp;
 	register struct conf_entry *ce;
@@ -588,12 +588,17 @@ request(conf)
 		n = select(sockfd + 1, &fdset, (fd_set *)0,
 		    (fd_set *)0, &tvout);
 
-		if (n <= 0) {
-			if (n < 0)
-				msyslog(LOG_ERR, "select() fails: %m");
-			return 0;
-		}
-
+		if (n < 0)
+		  {
+		    msyslog(LOG_ERR, "select() fails: %m");
+		    return 0;
+		  }
+		else if (n == 0)
+		  {
+		    if(debug)
+		      msyslog(LOG_DEBUG, "select() returned 0.");
+		    return 0;
+		  }
 
 #ifndef SYS_WINNT
 		n = recv(sockfd, (char *)&reqpkt, REQ_LEN_MAC, 0);
