@@ -76,10 +76,11 @@ u_fp clock_stability;		/* clock stability (ppm) */
 s_fp clock_frequency;		/* clock frequency error (ppm) */
 s_fp drift_comp;		/* pll frequency (ppm) */
 static long clock_adjust;	/* clock adjust (fraction only) */
+static int time_constant;	/* pll time constant */
 static s_fp max_comp;		/* max frequency offset (ppm) */
 int tc_counter;			/* poll-adjust counter */
 int pll_status;			/* status bits for kernel pll */
-VOLATILE int pll_control;	/* true if working kernel pll */
+volatile int pll_control;	/* true if working kernel pll */
 int pll_enable;			/* true if pll enabled */
 u_long pps_control;		/* last pps sample time */
 int pps_update;			/* pps update valid */
@@ -154,7 +155,11 @@ u_long tsf_limit;
 	 * maximum adjustment possible in adj_systime().
 	 */
 	max_comp = 0x7fff0000;
+#if defined(SCO3_TICKADJ) || defined(SCO5_TICKADJ)
+	tsf_limit = tsf_maxslew;
+#else
 	tsf_limit = ((tsf_maxslew >> 1) + (tsf_maxslew >> 2));
+#endif /* not SCO[35]_TICKADJ */
 	if ((max_comp >> RSH_DRIFT_TO_ADJ) > (s_fp) tsf_limit)
 		max_comp = tsf_limit << RSH_DRIFT_TO_ADJ;
 
@@ -187,8 +192,7 @@ local_clock(fp_offset, peer, fastset)
 	long tmp;
 	l_fp ftmp;
 	s_fp stmp;
-	u_long interval;
-	int time_constant;
+	long interval;
 #if defined(KERNEL_PLL)
 	struct timex ntv;
 #endif /* KERNEL_PLL */
@@ -196,7 +200,9 @@ local_clock(fp_offset, peer, fastset)
 	if (last_time == 0)
 		last_time = current_time;
 	interval = current_time - last_time;
-	time_constant = min(peer->ppoll, sys_poll) - NTP_MINPOLL;
+	 if (interval < 1)
+		interval = 1;
+	time_constant = min(peer->ppoll, sys_poll) - 2;
 	clock_adjust = 0;
 	offset = fp_offset->l_f;
 
@@ -225,14 +231,15 @@ local_clock(fp_offset, peer, fastset)
 	 * ms), reset the poll interval and wait for further
 	 * instructions. Note that the cutout switch is set when the
 	 * time is stepped, possibly because the frequency error is off
-	 * planet. In that case all sanity checks are disables and the
+	 * planet. In that case all sanity checks are disabled and the
 	 * discipine loop is on its own. Presumably, the loop will
 	 * eventually capture the wayward oscillator (if less than 500
 	 * ppm off planet) and converge, which will then reset the
 	 * cutout switch.
 	 */
-	} else if ((ftmp.l_ui > CLOCK_MAX_I || (ftmp.l_ui == CLOCK_MAX_I
-	    && ftmp.l_uf >= CLOCK_MAX_F)) && !cutout) {
+	} else if (ftmp.l_ui > CLOCK_MAX_I || ftmp.l_f < 0
+		   || (ftmp.l_ui == CLOCK_MAX_I && ftmp.l_uf >= CLOCK_MAX_F
+		       && !cutout)) {
 		tc_counter = 0;
 		sys_poll = peer->minpoll;
 
@@ -301,7 +308,7 @@ local_clock(fp_offset, peer, fastset)
 		}
 		ntv.esterror = sys_rootdispersion << 4;
 		ntv.maxerror = ntv.esterror + (sys_rootdelay << 2);
-		ntv.constant = min(peer->ppoll, sys_poll) - NTP_MINPOLL;
+		ntv.constant = min(peer->ppoll, sys_poll) - 2;
 		ntv.status = STA_PLL;
 		if (pps_enable)
 			ntv.status |= STA_PPSFREQ;
@@ -311,13 +318,12 @@ local_clock(fp_offset, peer, fastset)
 		/*
 		 * Set the leap bits in the status word.
 		 */
-		if (sys_leap & LEAP_ADDSECOND && sys_leap &
-		    LEAP_DELSECOND)
-		ntv.status |= STA_UNSYNC;
+		if (sys_leap & LEAP_ADDSECOND && sys_leap & LEAP_DELSECOND)
+			ntv.status |= STA_UNSYNC;
 		else if (sys_leap & LEAP_ADDSECOND)
-		ntv.status |= STA_INS;
+			ntv.status |= STA_INS;
 		else if (sys_leap & LEAP_DELSECOND)
-		ntv.status |= STA_DEL;
+			ntv.status |= STA_DEL;
 
 		/*
 		 * This astonishingly intricate wonder juggles the
@@ -404,17 +410,16 @@ local_clock(fp_offset, peer, fastset)
 	 * by the time constant, which is adjusted in response to the
 	 * phase error and dispersion.
 	 */ 
-	} else if (interval < CLOCK_MAXSEC || peer->maxpoll <=
-	    NTP_MAXDPOLL) {
-		u_long ltmp = interval;
+	} else if (interval < CLOCK_MAXSEC
+		   || peer->maxpoll <= NTP_MAXDPOLL) {
+		long ltmp = interval;
 
 		tmp = NTP_MAXDPOLL;
 		while (ltmp < (1 << NTP_MAXDPOLL)) {
 			tmp--;
 			ltmp <<= 1;
 		}
-		tmp = RSH_FRAC_TO_FREQ - tmp + time_constant +
-		    time_constant;
+		tmp = RSH_FRAC_TO_FREQ - tmp + time_constant + time_constant;
 		if (offset < 0)
 			drift_comp -= -offset >> tmp;
 		else
@@ -429,17 +434,14 @@ local_clock(fp_offset, peer, fastset)
 	 * factor of 4 in the left shift is to convert from s_fp to ppm.
 	 */
 	} else {
-		time_constant = NTP_MINDPOLL - NTP_MINPOLL;
+		time_constant = 2;
 		stmp = (offset / interval) << 4;
 		if (stmp < 0)
 			drift_comp -= -stmp >> CLOCK_G;
 		else
 			drift_comp += stmp >> CLOCK_G;
 	}
-	if (offset < 0)
-		clock_adjust = -(-offset >> time_constant);
-	else
-		clock_adjust = offset >> time_constant;
+	clock_adjust = offset;
 
 	/*
 	 * As a sanity check, we clamp the frequency not to exceed the
@@ -456,7 +458,7 @@ local_clock(fp_offset, peer, fastset)
 		stmp = -stmp;
 	if (stmp < CLOCK_MAX_FP)
 		cutout = 0;
-	if (interval > (u_long)(1 << (peer->minpoll - 1))) {
+	if (interval > (1 << (peer->minpoll - 1))) {
 
 		/*
 		 * Determine when to adjust the poll interval. We do
@@ -512,12 +514,12 @@ local_clock(fp_offset, peer, fastset)
 	if (debug > 1)
 		printf(
 		    "local_clock: phase %s freq %s disp %s poll %d count %d\n",
-		    mfptoa((clock_adjust < 0 ? -1 : 0), clock_adjust,
-		    6), fptoa(drift_comp, 3), fptoa(sys_maxd[0], 5),
+		    mfptoa((clock_adjust < 0 ? -1 : 0), clock_adjust, 6),
+		    fptoa(drift_comp, 3), fptoa(sys_maxd[0], 5),
 		    sys_poll, tc_counter);
 #endif /* DEBUG */
 
-	(void) record_loop_stats(fp_offset, drift_comp, sys_poll);
+	(void) record_loop_stats(fp_offset, drift_comp, (unsigned)sys_poll);
 	
 	/*
 	 * Whew. I've had enough.
@@ -564,9 +566,9 @@ adj_host_clock()
 		return;
 	adjustment = clock_adjust;
 	if (adjustment < 0)
-		adjustment = -(-adjustment >> CLOCK_PHASE);
+		adjustment = -(-adjustment >> (CLOCK_PHASE + time_constant));
 	else
-		adjustment >>= CLOCK_PHASE;
+		adjustment >>= CLOCK_PHASE + time_constant;
 	clock_adjust -= adjustment;
 	if (drift_comp < 0)
 		adjustment -= -drift_comp >> RSH_DRIFT_TO_ADJ;
@@ -607,10 +609,10 @@ adj_frequency(freq)
 	 * This routine adjusts the frequency offset. It is used by the
 	 * local clock driver to adjust frequency when no external
 	 * discipline source is available and by the acts driver when
-	 * the interval between updates is greater than 1 <<
-	 * NTP_MAXPOLL. Note that the maximum offset is limited by
-	 * max_comp when the daemon pll is used, but the maximum may be
-	 * different when the kernel pll is used.
+	 * the interval between updates is greater than 1 << NTP_MAXPOLL.
+	 * Note that the maximum offset is limited by max_comp when
+	 * the daemon pll is used, but the maximum may be different
+	 * when the kernel pll is used.
 	 */
 	drift_comp += freq;
 	if (drift_comp > max_comp)
@@ -678,7 +680,7 @@ loop_config(item, lfp_value)
 		ntv.maxerror = NTP_MAXDISPERSE;
 		ntv.esterror = NTP_MAXDISPERSE;
 		ntv.status = STA_PLL | STA_UNSYNC;
-		ntv.constant = sys_poll - NTP_MINPOLL;
+		ntv.constant = sys_poll - 2;
 #ifdef SIGSYS
 		newsigsys.sa_handler = pll_trap;
 		newsigsys.sa_flags = 0;
